<?php

namespace Drupal\epsilon_harmony\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Json;

/**
 * Implements the MessageConfiguration form controller.
 *
 * This example demonstrates a simple form with a singe text input element. We
 * extend FormBase which is the simplest form base class used in Drupal.
 *
 * @see \Drupal\Core\Form\FormBase
 */
class MessageConfiguration extends ConfigFormBase {

  const SETTINGS = 'epsilon_harmony.settings';

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * Getter method for Form ID.
   *
   * The form ID is used in implementations of hook_form_alter() to allow other
   * modules to alter the render array built by this form controller. It must be
   * unique site wide. It normally starts with the providing module's name.
   *
   * @return string
   *   The unique ID of the form defined by this class.
   */
  public function getFormId() {
    return 'epsilon_harmony_message_form';
  }

  /**
   * Build the simple form.
   *
   * A build form method constructs an array that defines how markup and
   * other form elements are included in an HTML form.
   *
   * @param array $form
   *   Default form array structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object containing current form state.
   *
   * @return array
   *   The render array defining the elements of the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);
    $form['#tree'] = TRUE;
    $initial_number = 0;

    // Gather the number of names in the form already.
    if (empty($form_state->get('available_message_fields'))) {
      $available_message_fields = $form_state->set('available_message_fields', 1);
    }

    for ($init = 1; $init <= $form_state->get('available_message_fields'); $init++) {
      $form['container'][$init] = array(
        '#type' => 'details',
        '#title' => $this->t('Message'),
        '#open' => TRUE
      );
      $form['container'][$init]['name'] = array(
        '#type' => 'textfield',
        '#title' => $this->t('Message Identifier'),
        '#required' => TRUE,
      );
      $form['container'][$init]['message_id'] = array(
        '#type' => 'textfield',
        '#title' => $this->t('Message ID'),
        '#required' => TRUE,
      );
    }

    $existing_message_array = $config->get('message_id_array');
    $message_array = JSON::decode($existing_message_array);
    $available_message_fields = count($message_array);

    $init = 1;
    if($available_message_fields > 0) {
      foreach ($message_array as $key => $value) {
        $form['container'][$init] = array(
          '#type' => 'details',
          '#title' => $this->t('Message: ' . $key),
          '#open' => FALSE
        );
        $form['container'][$init]['name'] = array(
          '#type' => 'textfield',
          '#title' => $this->t('Message Identifier'),
          '#disabled' => TRUE,
          '#default_value' => $key,
        );
        $form['container'][$init]['message_id'] = array(
          '#type' => 'textfield',
          '#title' => $this->t('Message ID'),
          '#default_value' => $value,
        );
        $init++;
      }
    }

    if ($available_message_fields > $form_state->get('available_message_fields')) {
      $form_state->set('available_message_fields', $available_message_fields);
    }

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['add_item'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Add another message ID'),
      '#submit' => array('::epsilon_harmony_message_form_add_item'),
    );

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    );
    return $form;
  }

  /**
   * Add a option to create a new message ID.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  function epsilon_harmony_message_form_add_item(array $form, FormStateInterface $form_state) {
    $current_count = $form_state->get('available_message_fields');
    $form_state->set('available_message_fields', $current_count + 1);
    $form_state->setRebuild(true);
  }

  /**
   * Implements a form submit handler.
   *
   * The submitForm method is the default method called for any submit elements.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValues()['container'] as $key => $value) {
      $message_id_array[$value['name']] = $value['message_id'];
    }
    // Set the submitted configuration setting
    $this->configFactory->getEditable(static::SETTINGS)
    ->set('message_id_array', JSON::encode($message_id_array))->save();

    $this->messenger()->addMessage($this->t('Your configurations have been saved.'));
  }
}
